/*
 * G-code meta compiler
 *
 * Copyright (C) 2013  B. Stultiens
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "config.h"

#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <assert.h>

#include "utils.h"

void testalloc(void **p, int n, int *na, size_t elemsize)
{
	if(!*p) {
		assert(n == 0);
		*p = calloc(8, elemsize);
		assert(*p != NULL);
		*na = 8;
	}
	if(n >= *na) {
		void *pp = calloc(2 * *na, elemsize);
		assert(pp != NULL);
		memcpy(pp, *p, n*elemsize);
		free(*p);
		*p = pp;
		*na *= 2;
	}
}

const char *planename(plane_et p)
{
	switch(p) {
	case PLANE_XY:	return "PLANE_XY";
	case PLANE_XZ:	return "PLANE_XZ";
	case PLANE_YZ:	return "PLANE_YZ";
	}
	return "<unknown plane>";
}

const char *unitshortname(unit_et u)
{
	switch(u) {
	case UNIT_NONE:	return "";
	case UNIT_MM:	return "mm";
	case UNIT_IN:	return "in";
	case UNIT_DEG:	return "deg";
	case UNIT_RAD:	return "rad";
	}
	return "<unknown units>";
}

const char *unitprintname(unit_et u)
{
	switch(u) {
	case UNIT_NONE:	return "none";
	case UNIT_MM:	return "mm";
	case UNIT_IN:	return "in";
	case UNIT_DEG:	return "degrees";
	case UNIT_RAD:	return "radians";
	}
	return "<unknown units>";
}

char *mysprintf(char **s, size_t *ns, const char *fmt, ...)
{
	int n;
	int size = 128;
	char *p;
	va_list va;

	p = malloc(size);
	assert(p != NULL);

	/* Adapted from snprintf(3) */
	while (1) {
		/* Try to print in the allocated space */
		va_start(va, fmt);
		n = vsnprintf(p, size, fmt, va);
		va_end(va);

		/* Break on error or when it fits */
		if(n < 0 || n < size)
			break;

		/* Else try again with more space */
		size = n + 1;       /* Precisely what is needed */

		p = realloc(p, size);
		assert(p != NULL);
	}
	assert(n >= 0);
	if(n > 0) {
		*s = realloc(*s, *ns + n + 1);
		assert(*s != NULL);
		*ns += n + 1;
		strcat(*s, p);
		free(p);
	}
	return *s;
}

